@@ -65,7 +65,10 @@ namespace GtkSharp.Generation {
 
 		public override string FromNative (string var, bool owned)
 		{
-			return "GLib.Object.GetObject (" + var + ", " + (owned ? "true" : "false") + ") as " + QualifiedName;
+			if (IsConsumeOnly)
+				return "GLib.Object.GetObject (" + var + ", " + (owned ? "true" : "false") + ") as " + QualifiedName;
+			else
+				return QualifiedName + "Adapter.GetObject (" + var + ", " + (owned ? "true" : "false") + ")";
 		}
 
 		public override bool ValidateForSubclass ()
@@ -103,7 +106,7 @@ namespace GtkSharp.Generation {
 			foreach (object member in members) {
 				if (member is Signal) {
 					Signal sig = member as Signal;
-					sw.WriteLine ("\t\t\tpublic IntPtr {0};", sig.ClassFieldName);
+					sw.WriteLine ("\t\t\tpublic IntPtr {0};", sig.CName.Replace ("\"", "").Replace ("-", "_"));
 				} else if (member is VirtualMethod) {
 					VirtualMethod vm = member as VirtualMethod;
 					bool has_target = methods [vm.Name] != null;
@@ -122,6 +125,7 @@ namespace GtkSharp.Generation {
 		{
 			sw.WriteLine ("\t\tstatic " + Name + "Adapter ()");
 			sw.WriteLine ("\t\t{");
+			sw.WriteLine ("\t\t\tGLib.GType.Register (_gtype, typeof({0}Adapter));", Name);
 			foreach (VirtualMethod vm in vms) {
 				bool has_target = methods [vm.Name] != null;
 				if (has_target && vm.IsValid)
@@ -169,15 +173,22 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\t\tthis.implementor = implementor;");
 			sw.WriteLine ("\t\t}");
 			sw.WriteLine ();
+			sw.WriteLine ("\t\tpublic " + Name + "Adapter (IntPtr handle)");
+			sw.WriteLine ("\t\t{");
+			sw.WriteLine ("\t\t\tthis.handle = handle;");
+			sw.WriteLine ("\t\t}");
+			sw.WriteLine ();
 		}
 
 		void GenerateGType (StreamWriter sw)
 		{
 			Method m = GetMethod ("GetType");
 			m.GenerateImport (sw);
+			sw.WriteLine ("\t\tprivate static GLib.GType _gtype = new GLib.GType ({0} ());", m.CName);
+			sw.WriteLine ();
 			sw.WriteLine ("\t\tpublic override GLib.GType GType {");
 			sw.WriteLine ("\t\t\tget {");
-			sw.WriteLine ("\t\t\t\treturn new GLib.GType (" + m.CName +  " ());");
+			sw.WriteLine ("\t\t\t\treturn _gtype;");
 			sw.WriteLine ("\t\t\t}");
 			sw.WriteLine ("\t\t}");
 			sw.WriteLine ();
@@ -185,8 +196,11 @@ namespace GtkSharp.Generation {
 
 		void GenerateHandleProp (StreamWriter sw)
 		{
+			sw.WriteLine ("\t\tIntPtr handle;");
 			sw.WriteLine ("\t\tpublic override IntPtr Handle {");
 			sw.WriteLine ("\t\t\tget {");
+			sw.WriteLine ("\t\t\t\tif (handle != IntPtr.Zero)");
+			sw.WriteLine ("\t\t\t\t\treturn handle;");
 			sw.WriteLine ("\t\t\t\treturn implementor == null ? IntPtr.Zero : implementor.Handle;");
 			sw.WriteLine ("\t\t\t}");
 			sw.WriteLine ("\t\t}");
@@ -198,8 +212,17 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\tpublic static " + Name + " GetObject (IntPtr handle, bool owned)");
 			sw.WriteLine ("\t\t{");
 			sw.WriteLine ("\t\t\tGLib.Object obj = GLib.Object.GetObject (handle, owned);");
-			sw.WriteLine ("\t\t\tif (obj is " + Name + "Implementor)");
+			sw.WriteLine ("\t\t\treturn GetObject (obj);");
+			sw.WriteLine ("\t\t}");
+			sw.WriteLine ();
+			sw.WriteLine ("\t\tpublic static " + Name + " GetObject (GLib.Object obj)");
+			sw.WriteLine ("\t\t{");
+			sw.WriteLine ("\t\t\tif (obj == null)");
+			sw.WriteLine ("\t\t\t\treturn null;");
+			sw.WriteLine ("\t\t\telse if (obj is " + Name + "Implementor)");
 			sw.WriteLine ("\t\t\t\treturn new {0}Adapter (obj as {0}Implementor);", Name);
+			sw.WriteLine ("\t\t\telse if (obj as " + Name + " == null)");
+			sw.WriteLine ("\t\t\t\treturn new {0}Adapter (obj.Handle);", Name);
 			sw.WriteLine ("\t\t\telse");
 			sw.WriteLine ("\t\t\t\treturn obj as {0};", Name);
 			sw.WriteLine ("\t\t}");
@@ -244,10 +267,8 @@ namespace GtkSharp.Generation {
 
 			GenProperties (gen_info, null);
 
-			foreach (Signal sig in sigs.Values) {
-				sig.GenCallback (sw);
+			foreach (Signal sig in sigs.Values)
 				sig.GenEvent (sw, null, "GLib.Object.GetObject (Handle)");
-			}
 
 			Method temp = methods ["GetType"] as Method;
 			if (temp != null)
@@ -314,6 +335,7 @@ namespace GtkSharp.Generation {
 
 		public override void Generate (GenerationInfo gen_info)
 		{
+			GenerateAdapter (gen_info);
 			StreamWriter sw = gen_info.Writer = gen_info.OpenStream (Name);
 
 			sw.WriteLine ("namespace " + NS + " {");
@@ -341,6 +363,7 @@ namespace GtkSharp.Generation {
 			AppendCustom (sw, gen_info.CustomDir);
 
 			sw.WriteLine ("\t}");
+			GenerateImplementorIface (sw);
 			sw.WriteLine ("#endregion");
 			sw.WriteLine ("}");
 			sw.Close ();
