@@ -28,39 +28,21 @@ namespace GtkSharp.Generation {
 	using System.Xml;
 
 	public class Method : MethodBase  {
-
+		
 		private ReturnValue retval;
 
 		private string call;
+		private string protection = "public";
 		private bool is_get, is_set;
 		private bool deprecated = false;
-		private bool win32_utf8_variant = false;
-		private bool cacheValue = false;
-		private string cacheName = string.Empty;
 
 		public Method (XmlElement elem, ClassBase container_type) : base (elem, container_type)
 		{
 			this.retval = new ReturnValue (elem["return-type"]);
-
-			if (!container_type.IsDeprecated && elem.HasAttribute ("deprecated")) {
-				string attr = elem.GetAttribute ("deprecated");
-				deprecated = attr == "1" || attr == "true";
-			}
-
-			if (elem.HasAttribute ("win32_utf8_variant")) {
-				string attr = elem.GetAttribute ("win32_utf8_variant");
-				win32_utf8_variant = attr == "1" || attr.ToLower () == "true";
-			}
-
-			if (Name == "GetType") {
+			if (!container_type.IsDeprecated && elem.HasAttribute ("deprecated"))
+				deprecated = elem.GetAttribute ("deprecated") == "1";
+			if (Name == "GetType")
 				Name = "GetGType";
-				cacheValue = !(container_type is StructBase);
-				cacheName = "_gtype";
-			}
-		}
-
-		public bool HasWin32Utf8Variant {
-			get { return win32_utf8_variant; }
 		}
 
 		public bool IsDeprecated {
@@ -81,6 +63,15 @@ namespace GtkSharp.Generation {
 			}
 		}
 
+		public string Protection {
+			get {
+				return protection;
+			}
+			set {
+				protection = value;
+			}
+		}
+
 		public string ReturnType {
 			get {
 				return retval.CSType;
@@ -102,7 +93,7 @@ namespace GtkSharp.Generation {
 
 			return true;
 		}
-
+		
 		private Method GetComplement ()
 		{
 			char complement;
@@ -110,10 +101,10 @@ namespace GtkSharp.Generation {
 				complement = 'S';
 			else
 				complement = 'G';
-
+			
 			return container_type.GetMethod (complement + BaseName.Substring (1));
 		}
-
+		
 		public string Declaration {
 			get {
 				return retval.CSType + " " + Name + " (" + (Signature != null ? Signature.ToString() : "") + ");";
@@ -171,7 +162,7 @@ namespace GtkSharp.Generation {
 				Method comp = GetComplement ();
 				if (comp != null && is_set)
 					return;
-
+			
 				sw.Write("\t\t");
 				GenerateDeclCommon (sw, null);
 
@@ -200,28 +191,12 @@ namespace GtkSharp.Generation {
 			string import_sig = IsStatic ? "" : container_type.MarshalType + " raw";
 			import_sig += !IsStatic && Parameters.Count > 0 ? ", " : "";
 			import_sig += Parameters.ImportSignature.ToString();
-			sw.WriteLine("\t\t[DllImport(\"" + LibraryName + "\", CallingConvention = CallingConvention.Cdecl)]");
+			sw.WriteLine("\t\t[DllImport(\"" + LibraryName + "\")]");
 			if (retval.MarshalType.StartsWith ("[return:"))
 				sw.WriteLine("\t\t" + retval.MarshalType + " static extern " + Safety + retval.CSType + " " + CName + "(" + import_sig + ");");
 			else
 				sw.WriteLine("\t\tstatic extern " + Safety + retval.MarshalType + " " + CName + "(" + import_sig + ");");
 			sw.WriteLine();
-
-			if (HasWin32Utf8Variant) {
-				sw.WriteLine("\t\t[DllImport(\"" + LibraryName + "\", CallingConvention = CallingConvention.Cdecl)]");
-				if (retval.MarshalType.StartsWith ("[return:"))
-					sw.WriteLine("\t\t" + retval.MarshalType + " static extern " + Safety + retval.CSType + " " + CName + "_utf8(" + import_sig + ");");
-				else
-					sw.WriteLine("\t\tstatic extern " + Safety + retval.MarshalType + " " + CName + "_utf8(" + import_sig + ");");
-				sw.WriteLine();
-			}
-		}
-
-		public void GenerateCacheValue (StreamWriter sw)
-		{
-			if (!cacheValue)
-				return;
-			sw.WriteLine ("\t\tstatic {0} {1} = new {0} ({2});", retval.CSType, cacheName, CName + call);
 		}
 
 		public void Generate (GenerationInfo gen_info, ClassBase implementor)
@@ -252,17 +227,16 @@ namespace GtkSharp.Generation {
 				if (comp != null && !comp.is_set)
 					comp = null;
 			}
-
+			
 			GenerateImport (gen_info.Writer);
-			GenerateCacheValue (gen_info.Writer);
 			if (comp != null && retval.CSType == comp.Parameters.AccessorReturnType)
 				comp.GenerateImport (gen_info.Writer);
 
 			if (IsDeprecated)
 				gen_info.Writer.WriteLine("\t\t[Obsolete]");
 			gen_info.Writer.Write("\t\t");
-			if (Protection != "")
-				gen_info.Writer.Write("{0} ", Protection);
+			if (protection != "")
+				gen_info.Writer.Write("{0} ", protection);
 			GenerateDeclCommon (gen_info.Writer, implementor);
 
 			if (is_get || is_set)
@@ -273,7 +247,7 @@ namespace GtkSharp.Generation {
 			}
 			else
 				GenerateBody (gen_info, implementor, "");
-
+			
 			if (is_get || is_set)
 			{
 				if (comp != null && retval.CSType == comp.Parameters.AccessorReturnType)
@@ -287,7 +261,7 @@ namespace GtkSharp.Generation {
 			}
 			else
 				gen_info.Writer.WriteLine();
-
+			
 			gen_info.Writer.WriteLine();
 
 			Statistics.MethodCount++;
@@ -303,31 +277,12 @@ namespace GtkSharp.Generation {
 				Body.InitAccessor (sw, Signature, indent);
 			Body.Initialize(gen_info, is_get, is_set, indent);
 
-			if (HasWin32Utf8Variant) {
-				if (!retval.IsVoid)
-					sw.WriteLine(indent + "\t\t\t" + retval.MarshalType + " raw_ret;");
-				sw.WriteLine(indent + "\t\t\t" + "if (Environment.OSVersion.Platform == PlatformID.Win32NT ||");
-				sw.WriteLine(indent + "\t\t\t" + "    Environment.OSVersion.Platform == PlatformID.Win32S ||");
-				sw.WriteLine(indent + "\t\t\t" + "    Environment.OSVersion.Platform == PlatformID.Win32Windows ||");
-				sw.WriteLine(indent + "\t\t\t" + "    Environment.OSVersion.Platform == PlatformID.WinCE)");
-				if (retval.IsVoid) {
-					sw.WriteLine(indent + "\t\t\t\t" + CName + "_utf8" + call + ";");
-					sw.WriteLine(indent + "\t\t\t" + "else");
-					sw.WriteLine(indent + "\t\t\t\t" + CName + call + ";");
-				} else {
-					sw.WriteLine(indent + "\t\t\t\traw_ret = " + CName + "_utf8" + call + ";");
-					sw.WriteLine(indent + "\t\t\t" + "else");
-					sw.WriteLine(indent + "\t\t\t\traw_ret = " + CName + call + ";");
-					sw.WriteLine(indent + "\t\t\t" + retval.CSType + " ret = " + retval.FromNative ("raw_ret") + ";");
-				}
-			} else {
-				sw.Write(indent + "\t\t\t");
-				if (retval.IsVoid)
-					sw.WriteLine(CName + call + ";");
-				else if (!cacheValue) {
-					sw.WriteLine(retval.MarshalType + " raw_ret = " + CName + call + ";");
-					sw.WriteLine(indent + "\t\t\t" + retval.CSType + " ret = " + retval.FromNative ("raw_ret") + ";");
-				}
+			sw.Write(indent + "\t\t\t");
+			if (retval.IsVoid)
+				sw.WriteLine(CName + call + ";");
+			else {
+				sw.WriteLine(retval.MarshalType + " raw_ret = " + CName + call + ";");
+				sw.WriteLine(indent + "\t\t\t" + retval.CSType + " ret = " + retval.FromNative ("raw_ret") + ";");
 			}
 
 			if (!IsStatic && implementor != null)
@@ -335,20 +290,21 @@ namespace GtkSharp.Generation {
 			Body.Finish (sw, indent);
 			Body.HandleException (sw, indent);
 
-			if (is_get && Parameters.Count > 0)
+			if (is_get && Parameters.Count > 0) 
 				sw.WriteLine (indent + "\t\t\treturn " + Parameters.AccessorName + ";");
-			else if (!retval.IsVoid) {
-				sw.WriteLine (indent + "\t\t\treturn " + (cacheValue ? cacheName : "ret") + ";");
-			} else if (IsAccessor)
+			else if (!retval.IsVoid)
+				sw.WriteLine (indent + "\t\t\treturn ret;");
+			else if (IsAccessor)
 				Body.FinishAccessor (sw, Signature, indent);
 
 			sw.Write(indent + "\t\t}");
 		}
 
-		bool IsAccessor {
-			get {
-				return retval.IsVoid && Signature.IsAccessor;
-			}
+		bool IsAccessor { 
+			get { 
+				return retval.IsVoid && Signature.IsAccessor; 
+			} 
 		}
 	}
 }
+
