@@ -85,36 +85,6 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		public string CallbackName {
-			get {
-				return Name + "SignalCallback";
-			}
-		}
-
-		private string CallbackSig {
-			get {
-				string result = "";
-				for (int i = 0; i < parms.Count; i++) {
-					if (i > 0)
-						result += ", ";
-
-					Parameter p = parms [i];
-					if (p.PassAs != "" && !(p.Generatable is StructBase))
-						result += p.PassAs + " ";
-					result += (p.NativeCallbackType + " arg" + i);
-				}
-				result += ", IntPtr gch";
-
-				return result;
-			}
-		}
-				
-		public string DelegateName {
-			get {
-				return Name + "SignalDelegate";
-			}
-		}
-
                 private string EventArgsName {
                         get {
                                 if (IsEventHandler)
@@ -153,6 +123,12 @@ namespace GtkSharp.Generation {
                         }
                 }
 
+		string ClassFieldName {
+			get {
+				return elem.HasAttribute ("field_name") ? elem.GetAttribute("field_name") : String.Empty;
+			}
+		}
+
 		private bool HasOutParams {
 			get {
 				foreach (Parameter p in parms) {
@@ -163,7 +139,7 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		public bool IsEventHandler {
+		private bool IsEventHandler {
 			get {
 				return retval.CSType == "void" && parms.Count == 1 && (parms [0].Generatable is ObjectGen || parms [0].Generatable is InterfaceGen);
 			}
@@ -199,73 +175,6 @@ namespace GtkSharp.Generation {
 			}
 		}
 
-		public void GenCallback (StreamWriter sw)
-		{
-			SymbolTable table = SymbolTable.Table;
-
-			sw.WriteLine ("\t\t[GLib.CDeclCallback]");
-			sw.WriteLine ("\t\tdelegate " + retval.ToNativeType + " " + DelegateName + " (" + CallbackSig + ");");
-			sw.WriteLine ();
-			sw.WriteLine ("\t\tstatic " + retval.ToNativeType + " " + CallbackName + " (" + CallbackSig + ")");
-			sw.WriteLine("\t\t{");
-			sw.WriteLine("\t\t\t{0} args = new {0} ();", EventArgsQualifiedName);
-			sw.WriteLine("\t\t\ttry {");
-			sw.WriteLine("\t\t\t\tGLib.Signal sig = ((GCHandle) gch).Target as GLib.Signal;");
-			sw.WriteLine("\t\t\t\tif (sig == null)");
-			sw.WriteLine("\t\t\t\t\tthrow new Exception(\"Unknown signal GC handle received \" + gch);");
-			sw.WriteLine();
-			if (parms.Count > 1)
-				sw.WriteLine("\t\t\t\targs.Args = new object[" + (parms.Count - 1) + "];");
-			string finish = "";
-			for (int idx = 1; idx < parms.Count; idx++) {
-				Parameter p = parms [idx];
-				IGeneratable igen = p.Generatable;
-				if (p.PassAs != "out") {
-					if (igen is ManualGen) {
-						sw.WriteLine("\t\t\t\tif (arg{0} == IntPtr.Zero)", idx);
-						sw.WriteLine("\t\t\t\t\targs.Args[{0}] = null;", idx - 1);
-						sw.WriteLine("\t\t\t\telse {");
-						sw.WriteLine("\t\t\t\t\targs.Args[" + (idx - 1) + "] = " + p.FromNative ("arg" + idx)  + ";");
-						sw.WriteLine("\t\t\t\t}");
-					} else
-						sw.WriteLine("\t\t\t\targs.Args[" + (idx - 1) + "] = " + p.FromNative ("arg" + idx)  + ";");
-				}
-				if (igen is StructBase && p.PassAs == "ref")
-					finish += "\t\t\t\tif (arg" + idx + " != IntPtr.Zero) System.Runtime.InteropServices.Marshal.StructureToPtr (args.Args[" + (idx-1) + "], arg" + idx + ", false);\n";
-				else if (p.PassAs != "")
-					finish += "\t\t\t\targ" + idx + " = " + igen.ToNativeReturn ("((" + p.CSType + ")args.Args[" + (idx - 1) + "])") + ";\n";
-			}
-			sw.WriteLine("\t\t\t\t{0} handler = ({0}) sig.Handler;", EventHandlerQualifiedName);
-			sw.WriteLine("\t\t\t\thandler (GLib.Object.GetObject (arg0), args);");
-			sw.WriteLine("\t\t\t} catch (Exception e) {");
-			sw.WriteLine("\t\t\t\tGLib.ExceptionManager.RaiseUnhandledException (e, false);");
-			sw.WriteLine("\t\t\t}");
-
-			if (IsVoid && finish.Length == 0) {
-				sw.WriteLine("\t\t}\n");
-				return;
-			}
-
-			sw.WriteLine("\n\t\t\ttry {");
-			sw.Write (finish);
-			if (!IsVoid) {
-				if (retval.CSType == "bool") {
-					sw.WriteLine ("\t\t\t\tif (args.RetVal == null)");
-					sw.WriteLine ("\t\t\t\t\treturn false;");
-				}
-				sw.WriteLine("\t\t\t\treturn " + table.ToNativeReturn (retval.CType, "((" + retval.CSType + ")args.RetVal)") + ";");
-			}
-			sw.WriteLine("\t\t\t} catch (Exception) {");
-			sw.WriteLine ("\t\t\t\tException ex = new Exception (\"args.RetVal or 'out' property unset or set to incorrect type in " + EventHandlerQualifiedName + " callback\");");
-			sw.WriteLine("\t\t\t\tGLib.ExceptionManager.RaiseUnhandledException (ex, true);");
-			
-			sw.WriteLine ("\t\t\t\t// NOTREACHED: above call doesn't return.");
-			sw.WriteLine ("\t\t\t\tthrow ex;");
-			sw.WriteLine("\t\t\t}");
-			sw.WriteLine("\t\t}");
-			sw.WriteLine();
-		}
-
 		private bool NeedNew (ClassBase implementor)
 		{
 			return elem.HasAttribute ("new_flag") ||
@@ -310,7 +219,7 @@ namespace GtkSharp.Generation {
 			sw.Close ();
 		}
 
-		private void GenVirtualMethod (StreamWriter sw, ClassBase implementor)
+		private void GenVMDeclaration (StreamWriter sw, ClassBase implementor)
 		{
 			VMSignature vmsig = new VMSignature (parms);
 			sw.WriteLine ("\t\t[GLib.DefaultSignalHandler(Type=typeof(" + (implementor != null ? implementor.QualifiedName : container_type.QualifiedName) + "), ConnectionMethod=\"Override" + Name +"\")]");
@@ -318,6 +227,101 @@ namespace GtkSharp.Generation {
 			if (NeedNew (implementor))
 				sw.Write ("new ");
 			sw.WriteLine ("virtual {0} {1} ({2})", retval.CSType, "On" + Name, vmsig.ToString ());
+		}
+
+		private string CastFromInt (string type)
+		{
+			return type != "int" ? "(" + type + ") " : "";
+		}
+
+		private string GlueCallString {
+			get {
+				string result = "Handle";
+
+				for (int i = 1; i < parms.Count; i++) {
+					Parameter p = parms [i];
+					IGeneratable igen = p.Generatable;
+
+					if (i > 1 && parms [i - 1].IsString && p.IsLength && p.PassAs == String.Empty) {
+						string string_name = parms [i - 1].Name;
+						result += ", " + igen.CallByName (CastFromInt (p.CSType) + "System.Text.Encoding.UTF8.GetByteCount (" +  string_name + ")");
+						continue;
+					}
+
+					p.CallName = p.Name;
+					string call_parm = p.CallString;
+
+					if (p.IsUserData && parms.IsHidden (p) && !parms.HideData && (i == 1 || parms [i - 1].Scope != "notified")) {
+						call_parm = "IntPtr.Zero"; 
+					}
+
+					result += ", " + call_parm;
+				}
+				return result;
+			}
+		}
+
+		private string GlueSignature {
+			get {
+				string result = String.Empty;
+				for (int i = 0; i < parms.Count; i++)
+					result += parms[i].CType.Replace ("const-", "const ") + " " + parms[i].Name + (i == parms.Count-1 ? "" : ", ");
+				return result;
+			}
+		}
+
+		private string DefaultGlueValue {
+			get {
+				string val = retval.DefaultValue;
+				switch (val) {
+				case "null":
+					return "NULL";
+				case "false":
+					return "FALSE";
+				case "true":
+					return "TRUE";
+				default:
+					return val;
+				}
+			}
+		}
+
+		private void GenGlueVirtualMethod (GenerationInfo gen_info)
+		{
+			StreamWriter glue = gen_info.GlueWriter;
+			string glue_name = String.Format ("{0}sharp_{1}_base_{2}", container_type.NS.ToLower ().Replace (".", "_"), container_type.Name.ToLower (), ClassFieldName);
+			glue.WriteLine ("{0} {1} ({2});\n", retval.CType, glue_name, GlueSignature);
+			glue.WriteLine ("{0}\n{1} ({2})", retval.CType, glue_name, GlueSignature);
+			glue.WriteLine ("{");
+			glue.WriteLine ("\t{0}Class *klass = ({0}Class *) get_threshold_class (G_OBJECT ({1}));", container_type.CName, parms[0].Name);
+			glue.Write ("\tif (klass->{0})\n\t\t", ClassFieldName);
+			if (!IsVoid)
+				glue.Write ("return ");
+			glue.Write ("(* klass->{0}) (", ClassFieldName);
+			for (int i = 0; i < parms.Count; i++)
+				glue.Write (parms[i].Name + (i == parms.Count - 1 ? "" : ", "));
+			glue.WriteLine (");");
+			if (!IsVoid)
+				glue.WriteLine ("\treturn " + DefaultGlueValue + ";");
+			glue.WriteLine ("}");
+
+			StreamWriter sw = gen_info.Writer;
+			sw.WriteLine ("\t\t[DllImport (\"{0}\")]", gen_info.GluelibName);
+			sw.WriteLine ("\t\tstatic extern {0} {1} ({2});\n", retval.MarshalType, glue_name, parms.ImportSignature);
+			GenVMDeclaration (sw, null);
+			sw.WriteLine ("\t\t{");
+			MethodBody body = new MethodBody (parms);
+			body.Initialize (gen_info, false, false, String.Empty);
+			sw.WriteLine ("\t\t\t{0}{1} ({2});", IsVoid ? "" : retval.MarshalType + " __ret = ", glue_name, GlueCallString);
+			body.Finish (sw, "");
+			if (!IsVoid)
+				sw.WriteLine ("\t\t\treturn {0};", retval.FromNative ("__ret"));
+			sw.WriteLine ("\t\t}\n");
+		}
+
+		private void GenChainVirtualMethod (StreamWriter sw, ClassBase implementor)
+		{
+			GenVMDeclaration (sw, implementor);
 			sw.WriteLine ("\t\t{");
 			if (IsVoid)
 				sw.WriteLine ("\t\t\tGLib.Value ret = GLib.Value.Empty;");
@@ -370,15 +374,37 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\t}\n");
 		}
 
-		private void GenDefaultHandlerDelegate (StreamWriter sw, ClassBase implementor)
+		private void GenDefaultHandlerDelegate (GenerationInfo gen_info, ClassBase implementor)
 		{
-			NativeCallbackSignature sig = new NativeCallbackSignature (parms);
-			ManagedCallString call = new ManagedCallString (parms);
+			StreamWriter sw = gen_info.Writer;
+			StreamWriter glue;
+			bool use_glue = gen_info.GlueEnabled && implementor == null && ClassFieldName.Length > 0;
+			string glue_name = String.Empty;
+			ManagedCallString call = new ManagedCallString (parms, true);
 			sw.WriteLine ("\t\t[GLib.CDeclCallback]");
-			sw.WriteLine ("\t\tdelegate " + retval.ToNativeType + " " + Name + "VMDelegate (" + sig.ToString () + ");\n");
+			sw.WriteLine ("\t\tdelegate " + retval.ToNativeType + " " + Name + "VMDelegate (" + parms.ImportSignature + ");\n");
+
+			if (use_glue) {
+				glue = gen_info.GlueWriter;
+				glue_name = String.Format ("{0}sharp_{1}_override_{2}", container_type.NS.ToLower ().Replace (".", "_"), container_type.Name.ToLower (), ClassFieldName);
+				sw.WriteLine ("\t\t[DllImport (\"{0}\")]", gen_info.GluelibName);
+				sw.WriteLine ("\t\tstatic extern void {0} (IntPtr gtype, {1}VMDelegate cb);\n", glue_name, Name);
+				glue.WriteLine ("void {0} (GType gtype, gpointer cb);\n", glue_name);
+				glue.WriteLine ("void\n{0} (GType gtype, gpointer cb)", glue_name);
+				glue.WriteLine ("{");
+				glue.WriteLine ("\tGObjectClass *klass = g_type_class_peek (gtype);");
+				glue.WriteLine ("\tif (klass == NULL)");
+				glue.WriteLine ("\t\tklass = g_type_class_ref (gtype);");
+				glue.WriteLine ("\t(({0} *)klass)->{1} = cb;", container_type.CName + "Class", ClassFieldName);
+				glue.WriteLine ("}\n");
+			}
+
 			sw.WriteLine ("\t\tstatic {0} {1};\n", Name + "VMDelegate", Name + "VMCallback");
-			sw.WriteLine ("\t\tstatic " + retval.ToNativeType + " " + Name.ToLower() + "_cb (" + sig.ToString () + ")");
+			sw.WriteLine ("\t\tstatic " + retval.ToNativeType + " " + Name.ToLower() + "_cb (" + parms.ImportSignature + ")");
 			sw.WriteLine ("\t\t{");
+			string unconditional = call.Unconditional ("\t\t\t");
+			if (unconditional.Length > 0)
+				sw.WriteLine (unconditional);
 			sw.WriteLine ("\t\t\ttry {");
 			sw.WriteLine ("\t\t\t\t{0} {1}_managed = GLib.Object.GetObject ({1}, false) as {0};", implementor != null ? implementor.Name : container_type.Name, parms[0].Name);
 			sw.Write (call.Setup ("\t\t\t\t"));
@@ -400,10 +426,34 @@ namespace GtkSharp.Generation {
 			sw.WriteLine ("\t\t{");
 			sw.WriteLine ("\t\t\tif (" + Name + "VMCallback == null)");
 			sw.WriteLine ("\t\t\t\t" + Name + "VMCallback = new " + Name + "VMDelegate (" + Name.ToLower() + "_cb);");
-			sw.WriteLine ("\t\t\tOverrideVirtualMethod (gtype, " + CName + ", " + Name + "VMCallback);");
+			if (use_glue)
+				sw.WriteLine ("\t\t\t{0} (gtype.Val, {1}VMCallback);", glue_name, Name);
+			else
+				sw.WriteLine ("\t\t\tOverrideVirtualMethod (gtype, " + CName + ", " + Name + "VMCallback);");
 			sw.WriteLine ("\t\t}\n");
 		}
 
+		public void GenEvent (StreamWriter sw, ClassBase implementor, string target)
+		{
+			string args_type = IsEventHandler ? "" : ", typeof (" + EventArgsQualifiedName + ")";
+
+			sw.WriteLine("\t\t[GLib.Signal("+ CName + ")]");
+			sw.Write("\t\tpublic ");
+			if (NeedNew (implementor))
+				sw.Write("new ");
+			sw.WriteLine("event " + EventHandlerQualifiedName + " " + Name + " {");
+			sw.WriteLine("\t\t\tadd {");
+			sw.WriteLine("\t\t\t\tGLib.Signal sig = GLib.Signal.Lookup (" + target + ", " + CName + args_type + ");");
+			sw.WriteLine("\t\t\t\tsig.AddDelegate (value);");
+			sw.WriteLine("\t\t\t}");
+			sw.WriteLine("\t\t\tremove {");
+			sw.WriteLine("\t\t\t\tGLib.Signal sig = GLib.Signal.Lookup (" + target + ", " + CName + args_type + ");");
+			sw.WriteLine("\t\t\t\tsig.RemoveDelegate (value);");
+			sw.WriteLine("\t\t\t}");
+			sw.WriteLine("\t\t}");
+			sw.WriteLine();
+		}
+
 		public void Generate (GenerationInfo gen_info, ClassBase implementor)
 		{
 			StreamWriter sw = gen_info.Writer;
@@ -411,27 +461,12 @@ namespace GtkSharp.Generation {
 			if (implementor == null)
 				GenEventHandler (gen_info);
 
-			if (!IsEventHandler)
-				GenCallback (sw);
-			GenDefaultHandlerDelegate (sw, implementor);
-			GenVirtualMethod (sw, implementor);
-			string marsh = IsEventHandler ? "" : ", new " + DelegateName + "(" + CallbackName + ")";
-
-			sw.WriteLine("\t\t[GLib.Signal("+ CName + ")]");
-			sw.Write("\t\tpublic ");
-			if (NeedNew (implementor))
-				sw.Write("new ");
-			sw.WriteLine("event " + EventHandlerQualifiedName + " " + Name + " {");
-			sw.WriteLine("\t\t\tadd {");
-			sw.WriteLine("\t\t\t\tGLib.Signal sig = GLib.Signal.Lookup (this, " + CName + marsh + ");");
-			sw.WriteLine("\t\t\t\tsig.AddDelegate (value);");
-			sw.WriteLine("\t\t\t}");
-			sw.WriteLine("\t\t\tremove {");
-			sw.WriteLine("\t\t\t\tGLib.Signal sig = GLib.Signal.Lookup (this, " + CName + marsh + ");");
-			sw.WriteLine("\t\t\t\tsig.RemoveDelegate (value);");
-			sw.WriteLine("\t\t\t}");
-			sw.WriteLine("\t\t}");
-			sw.WriteLine();
+			GenDefaultHandlerDelegate (gen_info, implementor);
+			if (gen_info.GlueEnabled && implementor == null && ClassFieldName.Length > 0)
+				GenGlueVirtualMethod (gen_info);
+			else
+				GenChainVirtualMethod (sw, implementor);
+			GenEvent (sw, implementor, "this");
 			
 			Statistics.SignalCount++;
 		}
